@@ -45,41 +45,29 @@ write_2s_texture <- function(piece_side = "tile_face", suit = 1, rank = 1, cfg =
                              ...,
                              filename = tempfile(fileext = ".png"), res = 72) {
 
-    height <- cfg$get_height(piece_side, suit, rank)
-    width <- cfg$get_width(piece_side, suit, rank)
+    piece <- get_piece(piece_side)
+    piece_face <- paste0(piece, "_face")
+    piece_back <- paste0(piece, "_back")
+    height <- cfg$get_height(piece_face, suit, rank)
+    width <- cfg$get_width(piece_face, suit, rank)
+    edge_color <- cfg$get_piece_opt(piece_face, suit, rank)$edge_color
+
     grDevices::png(filename, height = height, width = 2.5 * width,
         units = "in", res = res, bg = "transparent")
 
-    piece_face <- gsub("back", "face", piece_side)
-    piece_back <- gsub("face", "back", piece_side)
-
     # front
-    opt_face <- cfg$get_piece_opt(piece_face, suit, rank)
-    #### Update textures #206 #210
     pushViewport(viewport(x = 0.225, width = 0.45))
-    grid.rect(gp = gpar(col = "transparent", fill = opt_face$background_color))
-    popViewport()
-    pushViewport(viewport(x = 0.225, width = 0.40))
-    grid.piece(piece_face, suit, rank, cfg)
+    draw_piece_and_bleed(piece_face, suit, rank, cfg)
     popViewport()
 
     # edge
     pushViewport(viewport(x = 0.5, width = 0.1))
-    grid.rect(gp = gpar(col = "transparent", fill = opt_face$edge_color))
+    grid.rect(gp = gpar(col = "transparent", fill = edge_color))
     popViewport()
 
     # back
-    if (piece_side == "die_face") { #### proper 6-sided die OBJ files
-        piece_back <- "die_face"
-        rank <- 7 - rank
-    }
-    opt_back <- cfg$get_piece_opt(piece_back, suit, rank)
-    #### Update textures #206 #210
     pushViewport(viewport(x = 0.775, width = 0.45))
-    grid.rect(gp = gpar(col = "transparent", fill = opt_back$edge_color))
-    popViewport()
-    pushViewport(viewport(x = 0.775, width = 0.40))
-    grid.piece(piece_back, suit, rank, cfg)
+    draw_piece_and_bleed(piece_back, suit, rank, cfg)
     popViewport()
     grDevices::dev.off()
 
@@ -94,6 +82,37 @@ write_mtl <- function(mtl_filename, png_filename) {
 # 1,2,3,4 -> 1,4,3,2
 rev_shift <- function(x) c(x[1], rev(x[-1]))
 
+# 2-sided token angle
+side_R <- function(side) {
+    switch(side,
+           back = R_y(180),
+           base = R_x(-90),
+           left = R_y(-90) %*% R_z(-90),
+           right = R_y(90) %*% R_z(90),
+           top = R_x(90) %*% R_z(180),
+           diag(3))
+}
+
+get_scaling_factors <- function(side, width, height, depth) {
+    w <- switch(side,
+                left = depth,
+                right = depth,
+                width)
+    h <- switch(side,
+                back = height,
+                face = height,
+                left = width,
+                right = width,
+                depth)
+    d <- switch(side,
+                base = height,
+                top = height,
+                left = height,
+                right = height,
+                depth)
+    list(width = w, height = h, depth = d)
+}
+
 # two-sided objects
 write_2s_obj <- function(piece_side = "tile_face", suit = 1, rank = 1, cfg = pp_cfg(),
                          ...,
@@ -103,33 +122,26 @@ write_2s_obj <- function(piece_side = "tile_face", suit = 1, rank = 1, cfg = pp_
                          filename = tempfile(fileext = ".obj"), res = 72) {
 
     cfg <- as_pp_cfg(cfg)
-    opt <- cfg$get_piece_opt(piece_side, suit, rank)
+    piece <- get_piece(piece_side)
+    side <- get_side(piece_side)
+    opt <- cfg$get_piece_opt(paste0(piece, "_face"), suit, rank)
+    shape <- pp_shape(opt$shape, opt$shape_t, opt$shape_r, opt$back)
 
     # geometric vertices
-    # 1st half "top" vertices
-    # 2nd half "bottom" vertices
+    R <- side_R(side) %*% AA_to_R(angle, axis_x, axis_y)
+    whd <- get_scaling_factors(side, width, height, depth)
     pc <- Point3D$new(x, y, z)
-    xy_npc <- Point2D$new(get_shape_xy(opt$shape, opt$shape_t, opt$shape_r))
-    xy <- xy_npc$translate(-0.5, -0.5)
-    xy_reflected <- xy$dilate(width = -1, height = 1)
-    xy_npc_reflected <- xy_reflected$translate(0.5, 0.5)
-
-    xyz_t <- Point3D$new(xy, z = 0.5)
-    xyz_b <- Point3D$new(xy, z = -0.5)
-    xs <- c(xyz_t$x, xyz_b$x)
-    ys <- c(xyz_t$y, xyz_b$y)
-    zs <- c(xyz_t$z, xyz_b$z)
-    R <- AA_to_R(angle, axis_x, axis_y)
-    # If piece back then flip it over
-    if (grepl("_back", piece_side)) R <- R_y(180) %*% R
-    xyz <- Point3D$new(xs, ys, zs)$dilate(width, height, depth)$rotate(R)$translate(pc)
+    xyz <- Token2S$new(shape, whd, pc, R)$xyz
 
     # texture coordinates
-    xy_vt_t <- xy_npc$dilate(width = 0.4, height = 1)$translate(x = 0.025)
-    xy_vt_b <- xy_npc_reflected$dilate(width = 0.4, height = 1)$translate(x = 0.575)
+    back <- pp_shape(opt$shape, opt$shape_t, opt$shape_r, !opt$back)
+    xy_npc <- Point2D$new(shape$npc_coords)
+    xy_npc_back <- Point2D$new(back$npc_coords)
+    xy_vt_f <- xy_npc$dilate(width = 0.4, height = 1)$translate(x = 0.025)
+    xy_vt_b <- xy_npc_back$dilate(width = 0.4, height = 1)$translate(x = 0.575)
     xy_vt_e <- list(x = c(0.52, 0.48, 0.48, 0.52), y = c(0, 0, 1, 1))
-    vt <- list(x = c(xy_vt_t$x, xy_vt_b$x, xy_vt_e$x),
-               y =  c(xy_vt_t$y, xy_vt_b$y, xy_vt_e$y))
+    vt <- list(x = c(xy_vt_f$x, xy_vt_b$x, xy_vt_e$x),
+               y =  c(xy_vt_f$y, xy_vt_b$y, xy_vt_e$y))
 
     # textured face elements
     nv <- length(xyz) / 2
@@ -154,6 +166,56 @@ write_2s_obj <- function(piece_side = "tile_face", suit = 1, rank = 1, cfg = pp_
     invisible(list(obj = filename, mtl = mtl_filename, png = png_filename))
 }
 
+write_die_obj <- function(piece_side = "die_face", suit = 1, rank = 1, cfg = pp_cfg(),
+                          ...,
+                          x = 0, y = 0, z = 0,
+                          angle = 0, axis_x = 0, axis_y = 0,
+                          width = NA, height = NA, depth = NA,
+                         filename = tempfile(fileext = ".obj"), res = 72) {
+
+    cfg <- as_pp_cfg(cfg)
+    opt <- cfg$get_piece_opt(piece_side, suit, rank)
+    pc <- Point3D$new(x, y, z)
+
+    xs <- c(0, 0, 1, 1, 0, 0, 1, 1) - 0.5
+    ys <- c(1, 0, 0, 1, 1, 0, 0, 1) - 0.5
+    zs <- rep(c(1, 0), each = 4) - 0.5
+
+    # figure out rotation to reach each die face
+    rs <- get_die_face_info(suit, cfg$die_arrangement) #### also allow customization of angle #175
+    i <- which(rs$rank == rank)
+    dR <- switch(i,
+                 diag(3),
+                 R_y(-90) %*% R_z(90),
+                 R_x(90) %*% R_z(-90),
+                 R_x(180),
+                 R_y(90) %*% R_z(90),
+                 R_x(-90) %*% R_z(90),
+                 diag(3))
+    R <- dR %*% AA_to_R(angle, axis_x, axis_y)
+    xyz <- Point3D$new(xs, ys, zs)$dilate(width, height, depth)$rotate(R)$translate(pc)
+
+    xy_vt <- list(x = rep(c(0, 0.5, 1), 4),
+                  y = rep(c(1, 2/3, 1/3, 0), each = 3))
+
+    # textured face elements
+    f <- list()
+    f[[1]] <- list(v = 1:4, vt = c(1, 4, 5, 2))
+    f[[2]] <- list(v = c(8, 4, 3, 7), vt = c(2, 5, 6, 3))
+    f[[3]] <- list(v = c(1, 4, 8, 5), vt = c(4, 7, 8, 5))
+    f[[4]] <- list(v = c(6, 5, 8, 7), vt = c(5, 8, 9, 6))
+    f[[5]] <- list(v = c(1, 5, 6, 2), vt = c(7, 10, 11, 8))
+    f[[6]] <- list(v = c(3, 2, 6, 7), vt = c(8, 11, 12, 9))
+
+    ext <- tools::file_ext(filename)
+    mtl_filename <- gsub(paste0("\\.", ext, "$"), ".mtl", filename)
+    png_filename <- gsub(paste0("\\.", ext, "$"), ".png", filename)
+
+    write_obj(filename, v = xyz, vt = xy_vt, f = f)
+    write_die_texture(piece_side, suit, rank, cfg, filename = png_filename, res = res)
+
+    invisible(list(obj = filename, mtl = mtl_filename, png = png_filename))
+}
 
 write_pyramid_texture <- function(piece_side = "pyramid_face", suit = 1, rank = 1, cfg = pp_cfg(),
                              ...,
@@ -166,25 +228,64 @@ write_pyramid_texture <- function(piece_side = "pyramid_face", suit = 1, rank =
         units = "in", res = res, bg = "transparent")
 
     pushViewport(viewport(x = 0.125, width = 0.25))
-    grid.piece("pyramid_face", suit, rank, cfg)
+    draw_piece_and_bleed("pyramid_face", suit, rank, cfg)
     popViewport()
 
     pushViewport(viewport(x = 0.375, width = 0.25))
-    grid.piece("pyramid_left", suit, rank, cfg)
+    draw_piece_and_bleed("pyramid_left", suit, rank, cfg)
     popViewport()
 
     pushViewport(viewport(x = 0.625, width = 0.25))
-    grid.piece("pyramid_back", suit, rank, cfg)
+    draw_piece_and_bleed("pyramid_back", suit, rank, cfg)
     popViewport()
 
     pushViewport(viewport(x = 0.875, width = 0.25))
-    grid.piece("pyramid_right", suit, rank, cfg)
+    draw_piece_and_bleed("pyramid_right", suit, rank, cfg)
     popViewport()
-    grDevices::dev.off()
 
+    grDevices::dev.off()
     invisible(filename)
 }
 
+write_die_texture <- function(piece_side = "die_face", suit = 1, rank = 1, cfg = pp_cfg(),
+                              ..., filename = tempfile(fileext = ".png"), res = 72) {
+    width <- cfg$get_width("die_face", suit, rank)
+
+    grDevices::png(filename, height = 3 * width, width = 2 * width,
+                   units = "in", res = res, bg = "transparent")
+
+    rs <- get_die_face_info(suit, cfg$die_arrangement) #### also angle
+    pushViewport(viewport(x = 0.25, width = 0.5, y = 5/6, height = 1/3))
+    grid.piece("die_face", suit = rs$suit[1], rank = rs$rank[1], cfg = cfg)
+    popViewport()
+    pushViewport(viewport(x = 0.75, width = 0.5, y = 5/6, height = 1/3))
+    grid.piece("die_face", suit = rs$suit[2], rank = rs$rank[2], cfg = cfg)
+    popViewport()
+    pushViewport(viewport(x = 0.25, width = 0.5, y = 3/6, height = 1/3))
+    grid.piece("die_face", suit = rs$suit[3], rank = rs$rank[3], cfg = cfg)
+    popViewport()
+    pushViewport(viewport(x = 0.75, width = 0.5, y = 3/6, height = 1/3))
+    grid.piece("die_face", suit = rs$suit[4], rank = rs$rank[4], cfg = cfg)
+    popViewport()
+    pushViewport(viewport(x = 0.25, width = 0.5, y = 1/6, height = 1/3))
+    grid.piece("die_face", suit = rs$suit[5], rank = rs$rank[5], cfg = cfg)
+    popViewport()
+    pushViewport(viewport(x = 0.75, width = 0.5, y = 1/6, height = 1/3))
+    grid.piece("die_face", suit = rs$suit[6], rank = rs$rank[6], cfg = cfg)
+    popViewport()
+
+    grDevices::dev.off()
+    invisible(filename)
+}
+
+draw_piece_and_bleed <- function(piece_side, suit, rank, cfg) {
+    g <- pieceGrob(piece_side, suit, rank, cfg)
+    bleed_color <- cfg$get_piece_opt(piece_side, suit, rank)$bleed_color
+    b <- pp_shape()$polyclip(g, "minus", gp=gpar(col=NA, fill=bleed_color))
+    grid.draw(b)
+    grid.draw(g)
+}
+
 # pyramid top up
 write_pt_obj <- function(piece_side = "pyramid_top", suit = 1, rank = 1, cfg = pp_cfg(),
                          ...,
