@@ -5,6 +5,12 @@ test_that("options work as expected", {
     expect_equal(get_suit_color("coin_back", 1, 1, cfg), "white")
     expect_equal(get_suit_color("coin_face", 5, 1, cfg), "grey")
     expect_equal(pp_cfg(cfg)$get_suit_color(3:2), c("darkgreen", "black"))
+    cfg <- list(invert_colors.suited = TRUE,
+                background_color = "white",
+                suit_color = c("darkred", "black", "darkgreen", "darkblue", "grey"))
+    expect_equal(get_suit_color("coin_back", 1, 1, cfg), "white")
+    expect_equal(get_suit_color("coin_face", 5, 1, cfg), "grey")
+    expect_equal(pp_cfg(cfg)$get_suit_color(3:2), c("darkgreen", "black"))
 
     expect_equal(should_invert("coin_back", 1, 1, list()), FALSE)
     expect_equal(should_invert("coin_back", 1, 1, list(invert_colors = TRUE)), TRUE)
@@ -57,15 +63,16 @@ context("get_piece works as expected")
 test_that("get_piece_opt works as expected", {
     cfg <- pp_cfg(list(suit_color="darkred,black,darkgreen,darkblue,grey"))
     opt <- cfg$get_piece_opt("coin_face")
-    expect_equal(opt$background_col, "white")
-    expect_equal(opt$border_col, "grey")
-    expect_equal(opt$gridline_col, "transparent")
-    expect_equal(opt$ps_col, "grey")
+    expect_equal(opt$background_color, "white")
+    expect_equal(opt$border_color, "grey")
+    expect_equal(opt$bleed_color, "grey")
+    expect_equal(opt$gridline_color, "transparent")
+    expect_equal(opt$ps_color, "grey")
     expect_equal(opt$ps_text, "n")
     expect_equal(opt$ps_fontsize, 28)
     expect_equal(opt$ps_fontfamily, "sans")
     expect_equal(opt$ps_fontface, "plain")
-    expect_equal(opt$dm_col, "grey")
+    expect_equal(opt$dm_color, "grey")
     expect_equal(opt$dm_text, "\u25cf")
     expect_equal(opt$shape, "circle")
     expect_equal(opt$shape_r, 0.2)
@@ -96,6 +103,17 @@ test_that("get_piece_opt works as expected", {
     cfg <- pp_cfg(list(suit_color.s2="white"))
     opt <- cfg$get_piece_opt("tile_face", suit=2, rank=2)
     expect_equal(opt$ps_col, "white")
+
+    cfg <- pp_cfg(list(border_lex = 0))
+    expect_equal(cfg$get_piece_opt("tile_face")$bleed_color, "white")
+    cfg <- pp_cfg(list(border_color = "transparent"))
+    expect_equal(cfg$get_piece_opt("tile_face")$bleed_color, "white")
+    cfg <- pp_cfg(list(border_color = "transparent", mat_color = "blue"))
+    expect_equal(cfg$get_piece_opt("tile_face")$bleed_color, "white")
+    cfg <- pp_cfg(list(border_color = "transparent", mat_color = "blue", mat_width=0.1))
+    expect_equal(cfg$get_piece_opt("tile_face")$bleed_color, "blue")
+    cfg <- pp_cfg(list(border_color = "transparent", mat_color = "blue", background_color = "#00000000"))
+    expect_equal(cfg$get_piece_opt("tile_face")$bleed_color, "transparent")
 })
 
 context("pp_cfg querying variables work as expected")
